<?php

namespace Aplazame\Payment\Controller\Payment;

use Aplazame\Payment\Gateway\Config\Config;
use Aplazame\Payment\Model\Api\AplazameClient;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Api\CartManagementInterface;
use Magento\Quote\Model\QuoteManagement;
use Magento\Sales\Model\Order;

class Confirm extends Action
{
    /**
     * @var Session
     */
    private $checkoutSession;

    /**
     * @var AplazameClient
     */
    private $aplazameClient;

    /**
     * @var QuoteManagement
     */
    private $quoteManagement;

    /**
     * @var Config
     */
    private $config;

    public function __construct(
        Context $context,
        Session $checkoutSession,
        QuoteManagement $quoteManagement,
        AplazameClient $aplazameClient,
        Config $config
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->aplazameClient = $aplazameClient;
        $this->quoteManagement = $quoteManagement;
        $this->config = $config;

        parent::__construct($context);
    }

    public function execute()
    {
        $checkoutToken = $this->getRequest()->getParam('order_id');
        if (!$checkoutToken) {
            throw new LocalizedException(__('Confirm has no checkout token.'));
        }

        $quote = $this->checkoutSession->getQuote();
        if ($quote->getId() !== $checkoutToken) {
            throw new LocalizedException(__('Invalid token or session has expired.'));
        }

        if (empty($quote->getCustomerId())) {
            $quote->setCheckoutMethod(CartManagementInterface::METHOD_GUEST);
        }

        $this->quoteManagement->placeOrder($quote->getId());

        if ($this->config->shouldAutoInvoice()) {
            $order = $this->checkoutSession->getLastRealOrder();

            $this->invoice($order, $checkoutToken);
        }
    }

    private function invoice(Order $order, $checkoutToken)
    {
        $invoice = $order->prepareInvoice();

        $invoice
            ->setRequestedCaptureCase(\Magento\Sales\Model\Order\Invoice::NOT_CAPTURE)
            ->setTransactionId($checkoutToken)
            ->register();

        $this->_objectManager->create('Magento\Framework\DB\Transaction')
             ->addObject($invoice)
             ->addObject($order)
             ->save();

        $order->addRelatedObject($invoice);
        $payment = $order->getPayment();
        $payment->setCreatedInvoice($invoice);

        return $invoice;
    }
}
